/* utilities.c */

#include "config.h"

#include "utilities.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <err.h>

void *
my_calloc(size_t number, size_t size)
{
  void *buffer;

  buffer = calloc(number, size);
  if (buffer != NULL)
    return buffer;

  err(errno, "Out of memory");
}

void
my_free(char **buffer)
{
  if (*buffer == NULL)
    return;

  free(*buffer);
  *buffer = NULL;
}

char *
my_strdup(const char * const str)
{
  char *copy;

  copy = malloc(strlen(str)+1);
  if (copy == NULL)
    err(errno, "Out of memory");

  strcpy(copy, str);
  return copy;
}

int
is_fd_readable(int fd)
{
  int ret;
  fd_set readfds;
  struct timeval timeout;

  FD_ZERO(&readfds);
  FD_SET(fd, &readfds);
  timeout.tv_sec = 0;
  timeout.tv_usec = 0;
  ret = select(fd+1, &readfds, NULL, NULL, &timeout);

  if (ret < 0)
    return 0;

  if (FD_ISSET(fd, &readfds))
    return 1;

  return 0;
}

int
is_fd_writeable(int fd)
{
  int ret;
  fd_set writefds;
  struct timeval timeout;

  FD_ZERO(&writefds);
  FD_SET(fd, &writefds);
  timeout.tv_sec = 0;
  timeout.tv_usec = 0;
  ret = select(fd+1, NULL, &writefds, NULL, &timeout);

  if (ret < 0)
    return 0;

  if (FD_ISSET(fd, &writefds))
    return 1;

  return 0;
}

unsigned long
atoul(const char * const str)
{
  unsigned long num, temp;
  int i, j;
  if (str == NULL) return 0;

  num = 0;
  for (i=strlen(str)-1; i>=0; i--) {
    temp = (str[i]-'0');
    for (j=strlen(str)-1; j>i; j--)
      temp *= 10;
    num += temp;
  }
  return num;
}

/* eof */
